# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 0.43.09-v201909272304
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class OauthConfigurationOutputV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'client_id': 'str',
        'oauth_auto_redirect': 'bool',
        'oauth_enabled': 'bool',
        'tenant_id': 'str'
    }

    attribute_map = {
        'client_id': 'clientId',
        'oauth_auto_redirect': 'oauthAutoRedirect',
        'oauth_enabled': 'oauthEnabled',
        'tenant_id': 'tenantId'
    }

    def __init__(self, client_id=None, oauth_auto_redirect=False, oauth_enabled=False, tenant_id=None):
        """
        OauthConfigurationOutputV1 - a model defined in Swagger
        """

        self._client_id = None
        self._oauth_auto_redirect = None
        self._oauth_enabled = None
        self._tenant_id = None

        if client_id is not None:
          self.client_id = client_id
        if oauth_auto_redirect is not None:
          self.oauth_auto_redirect = oauth_auto_redirect
        if oauth_enabled is not None:
          self.oauth_enabled = oauth_enabled
        if tenant_id is not None:
          self.tenant_id = tenant_id

    @property
    def client_id(self):
        """
        Gets the client_id of this OauthConfigurationOutputV1.
        The client ID (AKA application ID, typically a GUID) of the Azure application. Found in the Microsoft Application Registration Portal.

        :return: The client_id of this OauthConfigurationOutputV1.
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """
        Sets the client_id of this OauthConfigurationOutputV1.
        The client ID (AKA application ID, typically a GUID) of the Azure application. Found in the Microsoft Application Registration Portal.

        :param client_id: The client_id of this OauthConfigurationOutputV1.
        :type: str
        """

        self._client_id = client_id

    @property
    def oauth_auto_redirect(self):
        """
        Gets the oauth_auto_redirect of this OauthConfigurationOutputV1.
        True if the Seeq sign-in page should auto-redirect to OAuth sign-in.

        :return: The oauth_auto_redirect of this OauthConfigurationOutputV1.
        :rtype: bool
        """
        return self._oauth_auto_redirect

    @oauth_auto_redirect.setter
    def oauth_auto_redirect(self, oauth_auto_redirect):
        """
        Sets the oauth_auto_redirect of this OauthConfigurationOutputV1.
        True if the Seeq sign-in page should auto-redirect to OAuth sign-in.

        :param oauth_auto_redirect: The oauth_auto_redirect of this OauthConfigurationOutputV1.
        :type: bool
        """
        if oauth_auto_redirect is None:
            raise ValueError("Invalid value for `oauth_auto_redirect`, must not be `None`")

        self._oauth_auto_redirect = oauth_auto_redirect

    @property
    def oauth_enabled(self):
        """
        Gets the oauth_enabled of this OauthConfigurationOutputV1.
        True if OAuth sign-in is available from the Seeq login screen.

        :return: The oauth_enabled of this OauthConfigurationOutputV1.
        :rtype: bool
        """
        return self._oauth_enabled

    @oauth_enabled.setter
    def oauth_enabled(self, oauth_enabled):
        """
        Sets the oauth_enabled of this OauthConfigurationOutputV1.
        True if OAuth sign-in is available from the Seeq login screen.

        :param oauth_enabled: The oauth_enabled of this OauthConfigurationOutputV1.
        :type: bool
        """
        if oauth_enabled is None:
            raise ValueError("Invalid value for `oauth_enabled`, must not be `None`")

        self._oauth_enabled = oauth_enabled

    @property
    def tenant_id(self):
        """
        Gets the tenant_id of this OauthConfigurationOutputV1.
        The tenant ID (typically a GUID) of the organization. Found in the Azure Portal.

        :return: The tenant_id of this OauthConfigurationOutputV1.
        :rtype: str
        """
        return self._tenant_id

    @tenant_id.setter
    def tenant_id(self, tenant_id):
        """
        Sets the tenant_id of this OauthConfigurationOutputV1.
        The tenant ID (typically a GUID) of the organization. Found in the Azure Portal.

        :param tenant_id: The tenant_id of this OauthConfigurationOutputV1.
        :type: str
        """

        self._tenant_id = tenant_id

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, OauthConfigurationOutputV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
